using System;
using System.Collections;
using System.Data;
using Microsoft.Practices.EnterpriseLibrary.Data;

namespace BookingApp.BLL
{
	/// <summary>
	/// This class provides an interface to two stored procedures for accessing flight data.
	/// For additional data access examples, see the UserRoleManager.cs class, which comes with Enterprise Library
	/// and is included in this solution.
	/// </summary>
	/// <notes></notes>
	public class Pricing
	{
		public Pricing()
		{
		}

		/// <summary>
		/// Retrieves a list of cities available for booking flights.
		/// </summary>
		/// <returns>An <see cref="ArrayList"/> containing the names of cities available.</returns>
		public ArrayList GetCities() {
			ArrayList cities = new ArrayList();
			// First, retrieve a reference to our Pricing database.  
			// Note that the only needed information is the name of the database instance.
			Database db = DatabaseFactory.CreateDatabase("Pricing");
			// Next, us an overloaded ExecuteReader to execute the GetCities stored procedure.
			// Because it takes no parameters, we can call the ExecuteReader method with just the name of the procedure.
			using (IDataReader reader = db.ExecuteReader("GetCities")) {
				while(reader.Read()) {
					cities.Add(reader.GetString(0));
				}
			}
		
			return cities;
		}

		/// <summary>
		/// Get's the price for a particular flight - the stored procedure handles the "days in advanced" issues.
		/// </summary>
		/// <param name="departureCity">The departure city.</param>
		/// <param name="arrivalCity">The arrival city.</param>
		/// <param name="flightDate">The date of the requested flight.</param>
		/// <returns></returns>
		public decimal GetPrice(string departureCity, string arrivalCity, DateTime flightDate) {
			// As before, get a reference to our database instance
			Database db = DatabaseFactory.CreateDatabase("Pricing");
			// The next several commands could have been accomplished with one line like:
			// object result = db.ExecuteScalar("GetPrice", departureCity, arrivalCity, flightDate);
			// But we're demonstrating the more complex version here in case you want to build a command separately
			// You may want to do this if you want to enroll the command in a transaction
			// Every DbCommandWrapper exposes a Command property, which gices you access to the transaction
			// First, ask the database instance for a "stored procedure command wrapper"
			DBCommandWrapper cmd = db.GetStoredProcCommandWrapper("GetPrice");
			// Set the parameter values.  Note that, although the example uses Sql Server, 
			// there are no "@" prefixes for the parameter names.  
			// This is because the DBCommandWrapper handles the prefixes automatically.
			cmd.AddInParameter("departureCity", DbType.String, departureCity);
			cmd.AddInParameter("arrivalCity", DbType.String, arrivalCity);
			cmd.AddInParameter("flightDate", DbType.String, flightDate);
			object result = db.ExecuteScalar(cmd);
			if (DBNull.Value == result) {
				throw new ApplicationException("No flights found matching the requested criteria.");
			}
			return Convert.ToDecimal(result);
		}
	}
}
